@@ -8,13 +8,21 @@ import psycopg2
 import sqlparse
 import pymysql
 import json
+import time
+import botocore.exceptions
 from .file import ConfigFile
+from .logger import Logger
 from psycopg2.extras import NamedTupleCursor
 from datetime import datetime
+from .extras import get_current_timestamp, MILLIS_FORMAT
+
+
+logging.getLogger('botocore').setLevel(logging.ERROR)
+logging.getLogger('boto3').setLevel(logging.ERROR)
 
 
 class MySQL(object):
-    def __init__(self, host, user, password, database, port, charset='UTF8', cursor_factory=pymysql.cursors.DictCursor, autocommit=True):
+    def __init__(self, host, user, password, database, port, charset='UTF8', cursor_factory=pymysql.cursors.DictCursor, autocommit=True, level='INFO'):
         """
         Constructor
         :param host: Host [String]
@@ -25,9 +33,9 @@ class MySQL(object):
         :param charset: Charset. Default UTF-8 [String]
         :param cursor_factory: Type of cursor, DictCursor or NamedTupleCursor (Default) [Cursor]
         :param autocommit: Enable or disable autocommit [Boolean]
-
+        :param level: logging level [String]
         """
-        self._logger = logging.getLogger(__name__)
+        self._logger = Logger(logger_name=__name__, level=level)
         settings = {"host": host,
                     "user": user,
                     "password": password,
@@ -315,7 +323,7 @@ class MySQL(object):
 
 class Redshift(object):
     def __init__(self, host, user, password, database, port, connect_timeout=20,
-                 cursor_factory=NamedTupleCursor, s3profile=None, autocommit=True):
+                 cursor_factory=NamedTupleCursor, s3profile=None, autocommit=True, level='INFO'):
         """
         Constructor
         :param host: Host [String]
@@ -325,8 +333,9 @@ class Redshift(object):
         :param port: Port number [Integer]
         :param connect_timeout: Timeout in seconds [Integer]
         :param cursor_factory: Type of cursor, DictCursor or NamedTupleCursor (Default) [Cursor]
+        :param level: logging level [String]
         """
-        self._logger = logging.getLogger(__name__)
+        self._logger = Logger(logger_name=__name__, level=level)
         settings = {"host": host,
                     "user": user,
                     "password": password,
@@ -572,7 +581,6 @@ class Redshift(object):
             cmd = os.linesep.join([s.strip() for s in cmd.splitlines() if s.strip()])
             return self.execute(cmd)
         except Exception as ex:
-            self._logger.error(repr(ex))
             raise
 
     def unload(self, prefix, query, compression=None, allowoverwrite=True,
@@ -672,70 +680,78 @@ class Redshift(object):
             cmd = os.linesep.join([s.strip() for s in cmd.splitlines() if s.strip()])
             return self.execute(cmd)
         except Exception as ex:
-            self._logger.error(repr(ex))
             raise
 
     def insert(self, into_table=None, from_table=None, **kwargs):
-            """
-            Generate INSERT statement
-            :param into_table: Target table name with schema [String]
-            :param from_table: From table name with schema [String]
-            :param kwargs: Fields mapping between target and from table, or mapping between columns and values [Dictionary]
-            :return: Data, Token and Rows affected [Dictionary]
-            """
-            try:
-                if not into_table:
-                    raise Exception('Missing target table')
+        """
+        Generate INSERT statement
+        :param into_table: Target table name with schema [String]
+        :param from_table: From table name with schema [String]
+        :param kwargs: Fields mapping between target and from table, or mapping between columns and values [Dictionary]
+        :return: Data, Token and Rows affected [Dictionary]
+        """
+        try:
+            if not into_table:
+                raise Exception('Missing target table')
 
-                # Listing values and fields
-                l_values = list()
-                l_fields = list()
+            # Listing values and fields
+            l_values = list()
+            l_fields = list()
 
-                if len(kwargs) == 0:
-                    cmd = 'INSERT INTO {into_table} '
-                    l_values.append('*')
-                else:
-                    cmd = 'INSERT INTO {into_table} ({into_fields}) '
-                    for arg in kwargs:
-                        l_fields.append(arg)
-                        value = str(kwargs[arg])
-                        if not from_table:
+            if len(kwargs) == 0:
+                cmd = 'INSERT INTO {into_table} '
+                l_values.append('*')
+            else:
+                cmd = 'INSERT INTO {into_table} ({into_fields}) '
+                for arg in kwargs:
+                    l_fields.append(arg)
+                    value = str(kwargs[arg])
+                    if not from_table:
+                        try:
+                            long(value)
+                        except Exception:
                             try:
-                                long(value)
+                                float(value)
                             except Exception:
-                                try:
-                                    float(value)
-                                except Exception:
-                                    value = "'{}'".format(value)
-                                    pass
+                                value = "'{}'".format(value)
                                 pass
-                            l_values.append(value)
-                        else:
-                            l_values.append(kwargs[arg])
+                            pass
+                        l_values.append(value)
+                    else:
+                        l_values.append(kwargs[arg])
 
-                if from_table:
-                    cmd += 'SELECT {from_fields} FROM {from_table};'
-                else:
-                    cmd += 'VALUES ({from_fields});'
+            if from_table:
+                cmd += 'SELECT {from_fields} FROM {from_table};'
+            else:
+                cmd += 'VALUES ({from_fields});'
 
-                into_fields = ', '.join(l_fields)
-                from_fields = ', '.join(l_values)
+            into_fields = ', '.join(l_fields)
+            from_fields = ', '.join(l_values)
 
-                cmd = cmd.format(into_table=into_table, into_fields=into_fields,
-                                 from_fields=from_fields, from_table=from_table)
+            cmd = cmd.format(into_table=into_table, into_fields=into_fields,
+                             from_fields=from_fields, from_table=from_table)
 
-                return self.execute(cmd)
-            except Exception as ex:
-                self._logger.error(repr(ex))
-                raise
+            return self.execute(cmd)
+        except Exception as ex:
+            raise
 
-    def drop(self, table):
+    def drop(self, table, table_type='table', cascade=True):
         """
         Drop Redshift table
         :param table: Table name with schema [String]
+        :param table_type: view or table [String]
+        :param cascade: enable cascade mode [Boolean]
         :return: Data, Token and Rows affected [Dictionary]
         """
-        cmd = "DROP TABLE IF EXISTS {};".format(table)
+        cmd = 'DROP TABLE IF EXISTS {}'.format(table)
+        if table_type == 'view':
+            cmd = 'DROP VIEW IF EXISTS {}'.format(table)
+
+        if cascade:
+            cmd += ' CASCADE;'
+        else:
+            cmd += ';'
+
         return self.execute(cmd)
 
     def truncate(self, table):
@@ -1035,9 +1051,21 @@ class Redshift(object):
             self._logger.debug('{} {}'.format(item['token'].upper(), item['rowcount']))
 
             return item
-        except Exception as ex:
-            self._logger.error(repr(ex))
-            exit(1)
+        except psycopg2.InternalError as ie:
+            lower_limit = ie.message.find("error") + 6
+            upper_limit = ie.message[lower_limit:].find('\n')
+            msg = ie.message[lower_limit:lower_limit + upper_limit].strip()
+
+            self._logger.error(msg)
+            raise psycopg2.InternalError(msg)
+        except psycopg2.ProgrammingError as pe:
+            self._logger.error(pe.message)
+            raise
+        except psycopg2.DataError as dd:
+            self._logger.error(dd.message)
+            raise
+        except psycopg2.DatabaseError as de:
+            self._logger.error(de.message)
             raise
 
     def parse_query(self, query):
@@ -1124,14 +1152,15 @@ class Redshift(object):
 
 
 class Athena(object):
-    def __init__(self, profile=None, database=None, output=None):
+    def __init__(self, profile=None, database=None, output=None, level='INFO'):
         """
         Constructor
         :param profile: AWS profile to get credentials from local configuration
         :param database: database name [String]
         :param output: S3 path (format s3://<bucket>/<prefix>) used for query output [String]
+        :param level: logging level [String]
         """
-        self._logger = logging.getLogger(__name__)
+        self._logger = Logger(logger_name=__name__, level=level)
         self._profile = profile
         self._database = None
         self._output = None
@@ -1210,7 +1239,7 @@ class Athena(object):
             self._logger.error(repr(ex))
             raise
 
-    def get_results(self, query_id, next_token=None, max_results=100):
+    def get_results(self, query_id, next_token=None, max_results=1000):
         """
         Get query results
         :param query_id: query identifier in Athena [String]
@@ -1218,11 +1247,10 @@ class Athena(object):
         :param max_results: the maximum number of results (rows) to return in this request [Integer]
         :return: query results [Dictionary]
         """
-        try:
-            sts = self.get_status(query_id)
-            if sts['QueryExecution']['Status']['State'] == 'RUNNING':
-                raise Exception('Query is running')
+        result = None
+        sts = self.get_status(query_id)
 
+        if sts['QueryExecution']['Status']['State'] != 'RUNNING':
             args = {
                     'QueryExecutionId': query_id,
                     'MaxResults': max_results
@@ -1231,10 +1259,9 @@ class Athena(object):
             if next_token:
                 args['NextToken'] = next_token
 
-            return self.client.get_query_results(**args)
-        except Exception as ex:
-            self._logger.error(repr(ex))
-            pass
+            result = self.client.get_query_results(**args)
+
+        return result
 
     def get_result_count(self, query_id):
         """
@@ -1267,14 +1294,15 @@ class Athena(object):
 
 
 class S3(object):
-    def __init__(self, bucket, profile=None, arn=None):
+    def __init__(self, bucket, profile=None, arn=None, level='INFO'):
         """
         Constructor
         :param bucket: Bucket name [String]
         :param profile: AWS profile to get credentials from local configuration
         :param arn: ARN role to assume [String]
+        :param level: logging level [String]
         """
-        self._logger = logging.getLogger(__name__)
+        self._logger = Logger(logger_name=__name__, level=level)
         self.profile = profile
         self.arn = arn
         self.bucket_name = bucket
@@ -1507,11 +1535,15 @@ class S3(object):
 
                 # Replace the prefix
                 suffix = obj.key.replace(source_prefix, destination_prefix)
-                suffix = sep.join([x for x in suffix.split(sep) if x.strip()])
 
-                destination_key = '/'.join([destination_prefix, suffix])
+                destination_key = sep.join([x for x in suffix.split(sep) if x.strip()])
                 destination_object = destination_bucket.Object(destination_key)
-                destination_object.copy(source_object)
+
+                extra_args = {
+                    'ACL': 'bucket-owner-full-control'
+                }
+
+                destination_object.copy(source_object, extra_args)
                 l_copied_objects.append(destination_key)
 
                 # Remove source object
@@ -1678,14 +1710,506 @@ class S3(object):
 
 
 class Neptune(object):
-    def __init__(self, endpoint):
+    def __init__(self, endpoint, level='INFO'):
         """
         Constructor
         """
 
 
 class Kinesis(object):
-    def __init__(self):
+    def __init__(self, aws_region, stream, shard_id, last_seq=None, arn=None, min_time_lag=5, max_stream_process_time_in_sec=600, level='INFO'):
         """
         Constructor
+        :param aws_region: aws region [String]
+        :param stream: kinesis stream name [String]
+        :param shard_id: shard identifier [String]
+        :param arn: iam role to assume [String]
+        :param level: logging level [String]
         """
+        self._logger = Logger(logger_name=__name__, level=level)
+        self._aws_region = aws_region
+        self._stream = stream
+        self._shard_id = shard_id
+        self._arn = arn
+        self._last_seq = last_seq
+        self._min_time_lag = min_time_lag
+        self._max_stream_process_time_in_sec = max_stream_process_time_in_sec
+
+        if self._arn:
+            response = boto3.client('sts').assume_role(RoleArn=self._arn, RoleSessionName='role')
+            self._logger.info('Kinesis client assumed role: {arn}'.format(arn=self._arn))
+
+            self._client = boto3.client('kinesis',
+                                        region_name=self._aws_region,
+                                        aws_access_key_id=response['Credentials']['AccessKeyId'],
+                                        aws_secret_access_key=response['Credentials']['SecretAccessKey'],
+                                        aws_session_token=response['Credentials']['SessionToken'])
+        else:
+            self._client = boto3.client('kinesis')
+
+    @staticmethod
+    def _is_record_empty(record):
+        """
+        Check if record is empty
+        :param record:
+        :return:
+        """
+        try:
+            json.loads(record[u'Data'])
+            return False
+        except ValueError:
+            return True
+
+    def set_last_sequence(self, seq):
+        """
+        Set last sequence read
+        :param seq: sequence number [Long]
+        :return: None
+        """
+        self._last_seq = seq
+
+    def keys_generator(self):
+        """
+        Get keys from Kinesis stream
+        :return: keys as iterator [Iterator]
+        """
+        if self._last_seq and str.isdigit(str(self._last_seq)):
+            self._logger.info('Fetching data starting from the sequence number: {}'.format(self._last_seq))
+            shard_iter = self._client.get_shard_iterator(
+                StreamName=self._stream,
+                ShardId=self._shard_id,
+                ShardIteratorType='AFTER_SEQUENCE_NUMBER',
+                StartingSequenceNumber=self._last_seq)
+
+        else:
+            self._logger.warning('Could not fetch the latest sequence number, will fetch from the earliest point available')
+
+            shard_iter = self._client.get_shard_iterator(
+                StreamName=self._stream,
+                ShardId=self._shard_id,
+                ShardIteratorType='TRIM_HORIZON')
+
+        assert shard_iter['ResponseMetadata']['HTTPStatusCode'] == 200
+        assert shard_iter['ShardIterator']
+
+        next_shard_iterator = shard_iter['ShardIterator']
+        consecutive_empty_shards = 0
+        time_streaming_start = time.time()
+
+        while True:
+            shard_iterator = next_shard_iterator
+            consecutive_empty_shards += 1
+
+            try:
+                records_response = self._client.get_records(ShardIterator=shard_iterator)
+            except botocore.exceptions.ClientError as e:
+                if e.response['Error']['Code'] != 'ProvisionedThroughputExceededException':
+                    raise e
+                self._logger.debug('Got ProvisionedThroughputExceededException - will sleep for 5 seconds')
+                time.sleep(5)
+                continue
+
+            if records_response is None:
+                self._logger.debug('ShardIterator was closed.')
+                break
+            else:
+                self._logger.debug('Got {} records ({} minutes behind latest)'.format(len(records_response['Records']), round(records_response['MillisBehindLatest'] / (1000 * 60))))
+
+            for record in records_response['Records']:
+                if self._is_record_empty(record):
+                    self._logger.debug('Empty record')
+                    break
+
+                sequence_number = record[u'SequenceNumber']
+                time_arrival = record[u'ApproximateArrivalTimestamp']
+                d = json.loads(record[u'Data'])
+                self._last_seq = sequence_number
+                yield dict(time_arrival=time_arrival,
+                           sequence=sequence_number,
+                           bucket=d['source_bucket'],
+                           key=d['file_name'])
+
+                consecutive_empty_shards = 0
+
+            next_shard_iterator = records_response.get('NextShardIterator', None)
+
+            if not next_shard_iterator:
+                self._logger.debug('Shutting down stream, REASON: exhaust stream shard iterator.')
+                break
+
+            timelag = round(records_response['MillisBehindLatest'] / 1000)
+            self._logger.debug('Timelag {time}'.format(time=timelag))
+
+            if timelag < self._min_time_lag:
+                self._logger.debug('Shutting down stream, REASON: stream events caught up with present.')
+                break
+            stream_processing_time = time.time() - time_streaming_start
+            self._logger.debug('Processing stream since {time}'.format(time=stream_processing_time))
+            if stream_processing_time > self._max_stream_process_time_in_sec:
+                self._logger.debug('Shutting down stream, REASON: reach max streaming duration.')
+                break
+
+
+class EMR(object):
+    def __init__(self, name, aws_region, applications, logs, bootstrap, script, emr_version, subnet_id, security_group,
+                 master_instance_type, master_instance_count, master_instance_volume_size, master_instance_volume_type, master_instance_volume_count,
+                 slave_instance_type=None, slave_instance_count=None, slave_instance_volume_size=None, slave_instance_volume_type=None, slave_instance_volume_count=None,
+                 key_name=None, arn=None, level='INFO'):
+        """
+        Constructor
+        :param name: cluster name [String]
+        :param aws_region: aws region of the cluster [String]
+        :param applications: list of applications to use in the EMR cluest [List<String>]
+        :param logs: S3 prefix to store the logs [String]
+        :param bootstrap: remote local path to bootstrap file [String]
+        :param script: remote local path to script to run [String]
+        :param emr_version: emr version [String]
+        :param subnet_id: subnet identifier [String]
+        :param security_group: security group [String]
+        :param master_instance_type: master instance type [String]
+        :param master_instance_count: number of master instances [Integer]
+        :param master_instance_volume_size: volume size in GB [Integer]
+        :param master_instance_volume_type: volume type [Integer]
+        :param master_instance_volume_count: volume count [Integer]
+        :param slave_instance_type: slave instance type [String]
+        :param slave_instance_count: number of slave instances [Integer]
+        :param slave_instance_volume_size: volume size in GB [Integer]
+        :param slave_instance_volume_type: volume type [Integer]
+        :param slave_instance_volume_count: volume count [Integer]
+        :param key_name: key to connect via SSH [String]
+        :param arn: aws iam role [String]
+        :param level: logging level [String]
+        """
+        # logger
+        self._logger = Logger(logger_name=__name__, level=level)
+
+        # cluster
+        self._created_at = None
+        self._cluster_id = None
+        self._cluster_name = name
+        self._aws_region = aws_region
+        self._applications = applications
+        self._logs = logs
+        self._bootstrap = bootstrap
+        self._script = script
+        self._emr_version = emr_version
+        self._subnet_id = subnet_id
+        self._security_group = security_group
+
+        self._master_instance_type = master_instance_type
+        self._master_instance_count = master_instance_count
+        self._master_instance_volume_size = master_instance_volume_size
+        self._master_instance_volume_type = master_instance_volume_type
+        self._master_instance_volume_count = master_instance_volume_count
+
+        self._slave_instance_type = slave_instance_type
+        self._slave_instance_count = slave_instance_count
+        self._slave_instance_volume_size = slave_instance_volume_size
+        self._slave_instance_volume_type = slave_instance_volume_type
+        self._slave_instance_volume_count = slave_instance_volume_count
+
+        self._key_name = key_name
+        self._arn = arn
+        self._steps = []
+
+        try:
+            if self._arn:
+                self.sts_client = boto3.client('sts')
+                assumed_role = self.sts_client.assume_role(
+                    RoleArn=self._arn,
+                    RoleSessionName='emr-session'
+                )
+
+                credentials = assumed_role['Credentials']
+                self.client = boto3.client(
+                    'emr',
+                    region_name=self._aws_region,
+                    aws_access_key_id=credentials['AccessKeyId'],
+                    aws_secret_access_key=credentials['SecretAccessKey'],
+                )
+                self._logger.debug('ARN = {}'.format(self._arn))
+            else:
+                self._client = boto3.client('emr', region_name=self._aws_region)
+
+        except Exception as ex:
+            self._logger.error(repr(ex))
+            raise
+
+    def create_cluster(self):
+        """
+        Create EMR cluster with the steps given
+        :return: cluster id [String]
+        """
+        if self._steps:
+            apps = []
+            for app in self._applications:
+                apps.append({'Name': app})
+
+            instance_groups = []
+            master_instance = {
+                "Name": "Master",
+                "InstanceRole": "MASTER",
+                "InstanceType": self._master_instance_type,
+                "InstanceCount": self._master_instance_count,
+                "EbsConfiguration": {
+                    "EbsBlockDeviceConfigs": [
+                        {
+                            "VolumeSpecification": {
+                                "SizeInGB": self._master_instance_volume_size,
+                                "VolumeType": self._master_instance_volume_type
+                            },
+                            "VolumesPerInstance": self._master_instance_volume_count
+                        }
+                    ]
+                }
+            }
+
+            instance_groups.append(master_instance)
+
+            if self._slave_instance_type:
+                slave_instance = {
+                    "Name": "Core",
+                    "InstanceRole": "CORE",
+                    "InstanceType": self._slave_instance_type,
+                    "InstanceCount": self._slave_instance_count,
+                    "EbsConfiguration": {
+                        "EbsBlockDeviceConfigs": [
+                            {
+                                "VolumeSpecification": {
+                                    "SizeInGB": self._slave_instance_volume_size,
+                                    "VolumeType": self._slave_instance_volume_type
+                                },
+                                "VolumesPerInstance": self._slave_instance_volume_count
+                            }
+                        ]
+                    }
+                }
+                instance_groups.append(slave_instance)
+
+            self._created_at = get_current_timestamp()
+
+            response = self._client.run_job_flow(
+                Name=self._cluster_name,
+                LogUri=self._logs,
+                ReleaseLabel=self._emr_version,
+                Applications=apps,
+                Instances={
+                    'InstanceGroups': instance_groups,
+                    'Ec2KeyName': self._key_name,
+                    'KeepJobFlowAliveWhenNoSteps': False,
+                    'TerminationProtected': False,
+                    'Ec2SubnetId': self._subnet_id,
+                    'EmrManagedMasterSecurityGroup': self._security_group,
+                    'EmrManagedSlaveSecurityGroup': self._security_group
+                },
+                Configurations=[
+                    {
+                        "Classification": "spark",
+                        "Properties": {
+                            "maximizeResourceAllocation": "true"
+                        }
+                    },
+                    {
+                        "Classification": "mapred-site",
+                        "Properties": {
+                            "mapred.output.committer.class": "org.apache.hadoop.mapred.FileOutputCommitter",
+                            "mapreduce.fileoutputcommitter.algorithm.version": "2"
+                        }
+                    }
+                ],
+
+                BootstrapActions=[
+                    {
+                        'Name': 'Configure Cluster',
+                        'ScriptBootstrapAction': {
+                            'Path': self._bootstrap
+                        }
+                    }
+                ],
+                Tags=[
+                    {
+                        'Key': 'application',
+                        'Value': 'amazonas'
+                    },
+                    {
+                        'Key': 'project',
+                        'Value': 'panamera'
+                    },
+                    {
+                        'Key': 'team',
+                        'Value': 'bi'
+                    }
+                ],
+                EbsRootVolumeSize=30,
+                ScaleDownBehavior='TERMINATE_AT_TASK_COMPLETION',
+                VisibleToAllUsers=True,
+                JobFlowRole='EMR_EC2_DefaultRole',
+                ServiceRole='EMR_DefaultRole',
+                Steps=self._steps
+            )
+            self._cluster_id = response['JobFlowId']
+
+        else:
+            self._logger.warning('No steps to run')
+
+        return self._cluster_id
+
+    def add_hadoop_step(self, name, args):
+        """
+        Add Hadoop step to run
+        :param name: step name [String]
+        :param args: list of args to run the script [List<String>]
+        :return:
+        """
+        step = {
+            'Name': name,
+            'ActionOnFailure': 'CONTINUE',
+            'HadoopJarStep': {
+                'Jar': 'command-runner.jar',
+                'Args': [
+                    'spark-submit',
+                    '--deploy-mode', 'client',
+                    '--master', 'yarn',
+                    self._script
+                ]
+            }
+        }
+        step['HadoopJarStep']['Args'] = step['HadoopJarStep']['Args'] + args
+        self._steps.append(step)
+
+    def get_steps_by_status(self, status):
+        """
+        Get steps with a given status
+        :param status: status name [String]
+        :return: list of steps with the status given [Dict]
+        """
+        steps = []
+        if self._cluster_id:
+            try:
+                response = self._client.list_steps(
+                    ClusterId=self._cluster_id
+                )
+
+                if 'Steps' in response:
+                    steps = response['Steps']
+
+                    for step in steps:
+                        if 'Status' in step:
+                            if 'State' in step['Status']:
+                                if step['Status']['State'] == status.upper():
+                                    steps += step
+
+            except Exception as _:
+                pass
+
+        return steps
+
+    def get_step_status(self, step_id):
+        """
+        Get step status
+        :param step_id: step identifier [String]
+        :return: status of the step [Dict]
+        """
+        status = None
+        if self._cluster_id:
+            try:
+                response = self._client.list_steps(
+                    ClusterId=self._cluster_id
+                )
+
+                if 'Steps' in response:
+                    steps = response['Steps']
+
+                    for step in steps:
+                        if step['Id'] == step_id:
+                            status = step
+                            break
+
+            except Exception as _:
+                pass
+
+        return status
+
+    def get_cluster_status(self):
+        """
+        Get cluster status
+        :return: cluster status State, Code and Message [Dictionary<String>]
+        """
+        status = {
+            'State': None,
+            'Code': None,
+            'Message': None
+        }
+
+        if self._cluster_id:
+            try:
+                response = self._client.list_clusters(
+                    CreatedAfter=self._created_at,
+                )
+                if 'Clusters' in response:
+                    for cluster in response['Clusters']:
+                        if cluster['Id'] == self._cluster_id:
+
+                            status['State'] = cluster['Status']['State']
+                            if 'Code' in cluster['Status']['StateChangeReason']:
+                                status['Code'] = cluster['Status']['StateChangeReason']['Code']
+
+                            if 'Message' in cluster['Status']['StateChangeReason']:
+                                status['Message'] = cluster['Status']['StateChangeReason']['Message']
+            except Exception as _:
+                pass
+
+        return status
+
+    def get_steps(self):
+        """
+        Get list of steps
+        :return: lists of steps [List<Dict>]
+        """
+        return self._steps
+
+    def count_steps(self):
+        """
+        Count number of steps
+        :return:
+        """
+        return len(self._steps)
+
+    def get_cluster_id(self):
+        """
+        Get cluster identifier
+        :return: cluster identifier [String]
+        """
+        return self._cluster_id
+
+    def get_step_id(self, step_name):
+        """
+        Get step identifier
+        :param step_name: step name [String]
+        :return: status of the step [Dict]
+        """
+        id = None
+        if self._cluster_id:
+            try:
+                response = self._client.list_steps(
+                    ClusterId=self._cluster_id
+                )
+
+                if 'Steps' in response:
+                    steps = response['Steps']
+
+                    for step in steps:
+                        if step['Name'] == step_name:
+                            id = step['Id']
+
+            except Exception as _:
+                pass
+
+        return id
+
+    def get_cluster_name(self):
+        """
+        Get the cluster name
+        :return: cluster name [String]
+        """
+        return self._cluster_name
