@@ -11,6 +11,17 @@ from email.mime.text import MIMEText
 from email.mime.multipart import MIMEMultipart
 from email.mime.application import MIMEApplication
 
+MILLIS_FORMAT = "%Y%m%d%H%M%S%f"
+DATE_FORMAT = "%Y-%m-%d"
+TS_MILLIS_FORMAT = "%Y-%m-%d %H:%M:%S.%f"
+TS_FORMAT = "%Y-%m-%d %H:%M:%S"
+HOUR_FORMAT = "%H"
+MINUTE_FORMAT = "%M"
+SECOND_FORMAT = "%S"
+EPOCH_FORMAT = "%s"
+MONTH_FORMAT = "%Y%m"
+YEAR_FORMAT = "%Y"
+
 logger = logging.getLogger(__name__)
 
 #####################################
@@ -102,6 +113,50 @@ MAP_CLASSIC_REGIONS = {
 }
 
 
+def is_valid_config(config, mandatory_keypaths):
+    """
+    Check if all mandatory keys are present in the configuration file
+    :param config: Configuration [JSONFile]
+    :param mandatory_keypaths: Mandatory keys to check [List<String>]
+    :return: Missing keys [List<String>]
+    """
+    missing_keypaths = set()
+
+    for keypath in mandatory_keypaths:
+        missing_keypaths.union(is_valid_path(config, keypath))
+
+    return missing_keypaths
+
+
+def is_valid_path(config, keypath, step=0):
+    """
+    Check recursively if the keypath exists in the configuration given
+    :param config: configuration object [List | Dict]
+    :param keypath: path to key [String]
+    :param step: index of keypath to check [Integer]
+    :return: set of missing keypaths in the configuration [Set<String>]
+    """
+    keys = keypath.split('/')
+    missing_keys = []
+
+    if step < len(keys):
+        key = keys[step]
+
+        if isinstance(config, dict):
+            if key in config:
+                missing_keys += is_valid_path(config[key], keypath, step + 1)
+            else:
+                missing_keys.append(keypath)
+        elif isinstance(config, list):
+            for e in config:
+                if key in e:
+                    missing_keys += is_valid_path(e[key], keypath, step + 1)
+                else:
+                    missing_keys.append(keypath)
+
+    return set(missing_keys)
+
+
 def get_business_region_for_classic_region(classic_region):
     """
     Get business region for classic region
@@ -320,6 +375,26 @@ def full_s3_path(bucket_name, prefix):
     return sep.join(['s3:/', bucket_name, prefix])
 
 
+def generate_manifest(bucket, keys):
+    """
+    Generate manifest with all prefixes
+    :param bucket: bucket name [String]
+    :param keys: lists of keys [List<String>]
+    :return: json manifest [Dictionary]
+
+    """
+    entries = []
+    for key in keys:
+        s3_path = full_s3_path(bucket_name=bucket, prefix=key)
+        entry = {
+            "url": s3_path,
+            "mandatory": True
+        }
+        entries.append(entry)
+
+    return {"entries": entries}
+
+
 #####################################
 #           OS Functions            #
 #####################################
